<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\DependencyInjection\Compiler;

use Dedi\SyliusMigrationPlugin\DependencyInjection\Configuration;
use Symfony\Component\Config\Definition\Processor;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;

class MigrationActionPass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container)
    {
        $actionRegistryDefinition = $container->getDefinition('dedi_migration_action_registry');

        $serviceIds = $container->findTaggedServiceIds('dedi_migration_action');
        foreach ($serviceIds as $serviceId => $tags) {
            $transformerDefinition = $container->getDefinition($serviceId);
            $actionRegistryDefinition->addMethodCall('addMigrationAction', [$transformerDefinition]);
        }

        $configuration = $container->getExtensionConfig('dedi_sylius_migration');
        $processor = new Processor();
        $mappingConfiguration = new Configuration();
        $configuration = $processor->processConfiguration(
            $mappingConfiguration,
            $configuration,
        );
        $actionConfigDefinition = $container->getDefinition('dedi_migration_action_config');

        if (!array_key_exists('migration_actions', $configuration)) {
            return;
        }

        foreach ($configuration['migration_actions'] as $type => $options) {
            if (count($options) === 0) {
                continue;
            }

            $actionConfigDefinition->addMethodCall(
                'addMigrationAction',
                [
                    $type,
                    $options,
                ],
            );
        }
    }
}
